@@ -5,45 +5,56 @@ import (
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
-	"net"
 	"os"
+	"strings"
 
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
-	ospclientconfig "github.com/gophercloud/utils/openstack/clientconfig"
+	igntypes "github.com/coreos/ignition/v2/config/v3_1/types"
 	gcpprovider "github.com/openshift/cluster-api-provider-gcp/pkg/apis/gcpprovider/v1beta1"
+	kubevirtprovider "github.com/openshift/cluster-api-provider-kubevirt/pkg/apis/kubevirtprovider/v1alpha1"
+	kubevirtutils "github.com/openshift/cluster-api-provider-kubevirt/pkg/utils"
 	libvirtprovider "github.com/openshift/cluster-api-provider-libvirt/pkg/apis/libvirtproviderconfig/v1beta1"
+	ovirtprovider "github.com/openshift/cluster-api-provider-ovirt/pkg/apis/ovirtprovider/v1beta1"
+	vsphereprovider "github.com/openshift/machine-api-operator/pkg/apis/vsphereprovider/v1beta1"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
-	awsprovider "sigs.k8s.io/cluster-api-provider-aws/pkg/apis/awsproviderconfig/v1beta1"
+	awsprovider "sigs.k8s.io/cluster-api-provider-aws/pkg/apis/awsprovider/v1beta1"
 	azureprovider "sigs.k8s.io/cluster-api-provider-azure/pkg/apis/azureprovider/v1beta1"
 	openstackprovider "sigs.k8s.io/cluster-api-provider-openstack/pkg/apis/openstackproviderconfig/v1alpha1"
 
 	"github.com/openshift/installer/pkg/asset"
 	"github.com/openshift/installer/pkg/asset/ignition"
 	"github.com/openshift/installer/pkg/asset/ignition/bootstrap"
+	baremetalbootstrap "github.com/openshift/installer/pkg/asset/ignition/bootstrap/baremetal"
 	"github.com/openshift/installer/pkg/asset/ignition/machine"
 	"github.com/openshift/installer/pkg/asset/installconfig"
-	azureconfig "github.com/openshift/installer/pkg/asset/installconfig/azure"
+	awsconfig "github.com/openshift/installer/pkg/asset/installconfig/aws"
 	gcpconfig "github.com/openshift/installer/pkg/asset/installconfig/gcp"
+	openstackconfig "github.com/openshift/installer/pkg/asset/installconfig/openstack"
+	ovirtconfig "github.com/openshift/installer/pkg/asset/installconfig/ovirt"
 	"github.com/openshift/installer/pkg/asset/machines"
 	"github.com/openshift/installer/pkg/asset/openshiftinstall"
 	"github.com/openshift/installer/pkg/asset/rhcos"
+	rhcospkg "github.com/openshift/installer/pkg/rhcos"
 	"github.com/openshift/installer/pkg/tfvars"
 	awstfvars "github.com/openshift/installer/pkg/tfvars/aws"
 	azuretfvars "github.com/openshift/installer/pkg/tfvars/azure"
 	baremetaltfvars "github.com/openshift/installer/pkg/tfvars/baremetal"
 	gcptfvars "github.com/openshift/installer/pkg/tfvars/gcp"
+	kubevirttfvars "github.com/openshift/installer/pkg/tfvars/kubevirt"
 	libvirttfvars "github.com/openshift/installer/pkg/tfvars/libvirt"
 	openstacktfvars "github.com/openshift/installer/pkg/tfvars/openstack"
+	ovirttfvars "github.com/openshift/installer/pkg/tfvars/ovirt"
+	vspheretfvars "github.com/openshift/installer/pkg/tfvars/vsphere"
 	"github.com/openshift/installer/pkg/types"
 	"github.com/openshift/installer/pkg/types/aws"
 	"github.com/openshift/installer/pkg/types/azure"
 	"github.com/openshift/installer/pkg/types/baremetal"
 	"github.com/openshift/installer/pkg/types/gcp"
+	"github.com/openshift/installer/pkg/types/kubevirt"
 	"github.com/openshift/installer/pkg/types/libvirt"
 	"github.com/openshift/installer/pkg/types/none"
 	"github.com/openshift/installer/pkg/types/openstack"
-	openstackdefaults "github.com/openshift/installer/pkg/types/openstack/defaults"
+	"github.com/openshift/installer/pkg/types/ovirt"
 	"github.com/openshift/installer/pkg/types/vsphere"
 )
 
@@ -84,6 +95,7 @@ func (t *TerraformVariables) Dependencies() []asset.Asset {
 		&machine.Master{},
 		&machines.Master{},
 		&machines.Worker{},
+		&baremetalbootstrap.IronicCreds{},
 	}
 }
 
@@ -98,11 +110,12 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 	workersAsset := &machines.Worker{}
 	rhcosImage := new(rhcos.Image)
 	rhcosBootstrapImage := new(rhcos.BootstrapImage)
-	parents.Get(clusterID, installConfig, bootstrapIgnAsset, masterIgnAsset, mastersAsset, workersAsset, rhcosImage, rhcosBootstrapImage)
+	ironicCreds := &baremetalbootstrap.IronicCreds{}
+	parents.Get(clusterID, installConfig, bootstrapIgnAsset, masterIgnAsset, mastersAsset, workersAsset, rhcosImage, rhcosBootstrapImage, ironicCreds)
 
 	platform := installConfig.Config.Platform.Name()
 	switch platform {
-	case none.Name, vsphere.Name:
+	case none.Name:
 		return errors.Errorf("cannot create the cluster because %q is a UPI platform", platform)
 	}
 
@@ -121,12 +134,22 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 		}
 	}
 
+	machineV4CIDRs, machineV6CIDRs := []string{}, []string{}
+	for _, network := range installConfig.Config.Networking.MachineNetwork {
+		if network.CIDR.IPNet.IP.To4() != nil {
+			machineV4CIDRs = append(machineV4CIDRs, network.CIDR.IPNet.String())
+		} else {
+			machineV6CIDRs = append(machineV6CIDRs, network.CIDR.IPNet.String())
+		}
+	}
+
 	masterCount := len(mastersAsset.MachineFiles)
 	data, err := tfvars.TFVars(
 		clusterID.InfraID,
 		installConfig.Config.ClusterDomain(),
 		installConfig.Config.BaseDomain,
-		&installConfig.Config.Networking.MachineNetwork[0].CIDR.IPNet,
+		machineV4CIDRs,
+		machineV6CIDRs,
 		useIPv4,
 		useIPv6,
 		bootstrapIgn,
@@ -178,6 +201,16 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			}
 		}
 
+		sess, err := installConfig.AWS.Session(ctx)
+		if err != nil {
+			return err
+		}
+		object := "bootstrap.ign"
+		bucket := fmt.Sprintf("%s-bootstrap", clusterID.InfraID)
+		url, err := awsconfig.PresignedS3URL(sess, installConfig.Config.Platform.AWS.Region, bucket, object)
+		if err != nil {
+			return err
+		}
 		masters, err := mastersAsset.Machines()
 		if err != nil {
 			return err
@@ -194,7 +227,26 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 		for i, m := range workers {
 			workerConfigs[i] = m.Spec.Template.Spec.ProviderSpec.Value.Object.(*awsprovider.AWSMachineProviderConfig)
 		}
-		data, err := awstfvars.TFVars(vpc, privateSubnets, publicSubnets, installConfig.Config.Publish, masterConfigs, workerConfigs)
+		osImage := strings.SplitN(string(*rhcosImage), ",", 2)
+		osImageID := osImage[0]
+		osImageRegion := installConfig.Config.AWS.Region
+		if len(osImage) == 2 {
+			osImageRegion = osImage[1]
+		}
+		data, err := awstfvars.TFVars(awstfvars.TFVarsSources{
+			VPC:                   vpc,
+			PrivateSubnets:        privateSubnets,
+			PublicSubnets:         publicSubnets,
+			Services:              installConfig.Config.AWS.ServiceEndpoints,
+			Publish:               installConfig.Config.Publish,
+			MasterConfigs:         masterConfigs,
+			WorkerConfigs:         workerConfigs,
+			AMIID:                 osImageID,
+			AMIRegion:             osImageRegion,
+			IgnitionBucket:        bucket,
+			IgnitionPresignedURL:  url,
+			AdditionalTrustBundle: installConfig.Config.AdditionalTrustBundle,
+		})
 		if err != nil {
 			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
 		}
@@ -203,15 +255,16 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			Data:     data,
 		})
 	case azure.Name:
-		sess, err := azureconfig.GetSession()
+		session, err := installConfig.Azure.Session()
 		if err != nil {
 			return err
 		}
+
 		auth := azuretfvars.Auth{
-			SubscriptionID: sess.Credentials.SubscriptionID,
-			ClientID:       sess.Credentials.ClientID,
-			ClientSecret:   sess.Credentials.ClientSecret,
-			TenantID:       sess.Credentials.TenantID,
+			SubscriptionID: session.Credentials.SubscriptionID,
+			ClientID:       session.Credentials.ClientID,
+			ClientSecret:   session.Credentials.ClientSecret,
+			TenantID:       session.Credentials.TenantID,
 		}
 		masters, err := mastersAsset.Machines()
 		if err != nil {
@@ -230,30 +283,19 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			workerConfigs[i] = w.Spec.Template.Spec.ProviderSpec.Value.Object.(*azureprovider.AzureMachineProviderSpec)
 		}
 
-		var (
-			machineV4CIDRs []net.IPNet
-			machineV6CIDRs []net.IPNet
-		)
-		for _, network := range installConfig.Config.Networking.MachineNetwork {
-			if network.CIDR.IPNet.IP.To4() != nil {
-				machineV4CIDRs = append(machineV4CIDRs, network.CIDR.IPNet)
-			} else {
-				machineV6CIDRs = append(machineV6CIDRs, network.CIDR.IPNet)
-			}
-		}
-
 		preexistingnetwork := installConfig.Config.Azure.VirtualNetwork != ""
 		data, err := azuretfvars.TFVars(
 			azuretfvars.TFVarsSources{
 				Auth:                        auth,
+				CloudName:                   installConfig.Config.Azure.CloudName,
+				ResourceGroupName:           installConfig.Config.Azure.ResourceGroupName,
 				BaseDomainResourceGroupName: installConfig.Config.Azure.BaseDomainResourceGroupName,
 				MasterConfigs:               masterConfigs,
 				WorkerConfigs:               workerConfigs,
 				ImageURL:                    string(*rhcosImage),
 				PreexistingNetwork:          preexistingnetwork,
 				Publish:                     installConfig.Config.Publish,
-				MachineV4CIDRs:              machineV4CIDRs,
-				MachineV6CIDRs:              machineV6CIDRs,
+				OutboundType:                installConfig.Config.Azure.OutboundType,
 			},
 		)
 		if err != nil {
@@ -295,12 +337,18 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			publicZoneName = publicZone.Name
 		}
 		preexistingnetwork := installConfig.Config.GCP.Network != ""
+
+		imageRaw, err := rhcospkg.GCPRaw(ctx, installConfig.Config.ControlPlane.Architecture)
+		if err != nil {
+			return errors.Wrap(err, "failed to find Raw GCP image URL")
+		}
 		data, err := gcptfvars.TFVars(
 			gcptfvars.TFVarsSources{
 				Auth:               auth,
 				MasterConfigs:      masterConfigs,
 				WorkerConfigs:      workerConfigs,
-				ImageURI:           string(*rhcosImage),
+				ImageURI:           imageRaw,
+				ImageLicenses:      installConfig.Config.GCP.Licenses,
 				PublicZoneName:     publicZoneName,
 				PublishStrategy:    installConfig.Config.Publish,
 				PreexistingNetwork: preexistingnetwork,
@@ -324,6 +372,7 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			&installConfig.Config.Networking.MachineNetwork[0].CIDR.IPNet,
 			installConfig.Config.Platform.Libvirt.Network.IfName,
 			masterCount,
+			installConfig.Config.ControlPlane.Architecture,
 		)
 		if err != nil {
 			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
@@ -333,15 +382,13 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			Data:     data,
 		})
 	case openstack.Name:
-		opts := &ospclientconfig.ClientOpts{}
-		opts.Cloud = installConfig.Config.Platform.OpenStack.Cloud
-		cloud, err := ospclientconfig.GetCloudFromYAML(opts)
+		cloud, err := openstackconfig.GetSession(installConfig.Config.Platform.OpenStack.Cloud)
 		if err != nil {
 			return errors.Wrap(err, "failed to get cloud config for openstack")
 		}
 		var caCert string
 		// Get the ca-cert-bundle key if there is a value for cacert in clouds.yaml
-		if caPath := cloud.CACertFile; caPath != "" {
+		if caPath := cloud.CloudConfig.CACertFile; caPath != "" {
 			caFile, err := ioutil.ReadFile(caPath)
 			if err != nil {
 				return errors.Wrap(err, "failed to read clouds.yaml ca-cert from disk")
@@ -353,33 +400,27 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 		if err != nil {
 			return err
 		}
-		apiVIP, err := openstackdefaults.APIVIP(installConfig.Config.Networking)
-		if err != nil {
-			return err
-		}
-		dnsVIP, err := openstackdefaults.DNSVIP(installConfig.Config.Networking)
-		if err != nil {
-			return err
-		}
-		ingressVIP, err := openstackdefaults.IngressVIP(installConfig.Config.Networking)
-		if err != nil {
-			return err
+
+		var masterSpecs []*openstackprovider.OpenstackProviderSpec
+		for _, master := range masters {
+			masterSpecs = append(masterSpecs, master.Spec.ProviderSpec.Value.Object.(*openstackprovider.OpenstackProviderSpec))
 		}
 		data, err = openstacktfvars.TFVars(
-			masters[0].Spec.ProviderSpec.Value.Object.(*openstackprovider.OpenstackProviderSpec),
+			masterSpecs,
 			installConfig.Config.Platform.OpenStack.Cloud,
 			installConfig.Config.Platform.OpenStack.ExternalNetwork,
 			installConfig.Config.Platform.OpenStack.ExternalDNS,
-			installConfig.Config.Platform.OpenStack.LbFloatingIP,
-			apiVIP.String(),
-			dnsVIP.String(),
-			ingressVIP.String(),
-			installConfig.Config.Platform.OpenStack.TrunkSupport,
-			installConfig.Config.Platform.OpenStack.OctaviaSupport,
+			installConfig.Config.Platform.OpenStack.APIFloatingIP,
+			installConfig.Config.Platform.OpenStack.IngressFloatingIP,
+			installConfig.Config.Platform.OpenStack.APIVIP,
+			installConfig.Config.Platform.OpenStack.IngressVIP,
 			string(*rhcosImage),
+			installConfig.Config.Platform.OpenStack.ClusterOSImageProperties,
 			clusterID.InfraID,
 			caCert,
 			bootstrapIgn,
+			installConfig.Config.ControlPlane.Platform.OpenStack,
+			installConfig.Config.Platform.OpenStack.MachinesSubnet,
 		)
 		if err != nil {
 			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
@@ -389,14 +430,24 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			Data:     data,
 		})
 	case baremetal.Name:
+		provisioningIP := installConfig.Config.Platform.BareMetal.BootstrapProvisioningIP
+		if installConfig.Config.Platform.BareMetal.ProvisioningNetwork == baremetal.DisabledProvisioningNetwork && provisioningIP == "" {
+			provisioningIP = installConfig.Config.Platform.BareMetal.APIVIP
+		}
+
 		data, err = baremetaltfvars.TFVars(
 			installConfig.Config.Platform.BareMetal.LibvirtURI,
-			installConfig.Config.Platform.BareMetal.BootstrapProvisioningIP,
+			provisioningIP,
 			string(*rhcosBootstrapImage),
 			installConfig.Config.Platform.BareMetal.ExternalBridge,
+			installConfig.Config.Platform.BareMetal.ExternalMACAddress,
 			installConfig.Config.Platform.BareMetal.ProvisioningBridge,
+			installConfig.Config.Platform.BareMetal.ProvisioningMACAddress,
 			installConfig.Config.Platform.BareMetal.Hosts,
 			string(*rhcosImage),
+			ironicCreds.Username,
+			ironicCreds.Password,
+			masterIgn,
 		)
 		if err != nil {
 			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
@@ -405,6 +456,115 @@ func (t *TerraformVariables) Generate(parents asset.Parents) error {
 			Filename: fmt.Sprintf(TfPlatformVarsFileName, platform),
 			Data:     data,
 		})
+	case ovirt.Name:
+		config, err := ovirtconfig.NewConfig()
+		if err != nil {
+			return err
+		}
+		con, err := ovirtconfig.NewConnection()
+		if err != nil {
+			return err
+		}
+		defer con.Close()
+
+		if installConfig.Config.Platform.Ovirt.VNICProfileID == "" {
+			profiles, err := ovirtconfig.FetchVNICProfileByClusterNetwork(
+				con,
+				installConfig.Config.Platform.Ovirt.ClusterID,
+				installConfig.Config.Platform.Ovirt.NetworkName)
+			if err != nil {
+				return errors.Wrapf(err, "failed to compute values for Engine platform")
+			}
+			if len(profiles) != 1 {
+				return errors.Wrapf(err, "failed to compute values for Engine platform, there are multiple vNIC profiles.")
+			}
+			installConfig.Config.Platform.Ovirt.VNICProfileID = profiles[0].MustId()
+		}
+
+		masters, err := mastersAsset.Machines()
+		if err != nil {
+			return err
+		}
+
+		data, err := ovirttfvars.TFVars(
+			ovirttfvars.Auth{
+				URL:      config.URL,
+				Username: config.Username,
+				Password: config.Password,
+				Cafile:   config.CAFile,
+			},
+			installConfig.Config.Platform.Ovirt.ClusterID,
+			installConfig.Config.Platform.Ovirt.StorageDomainID,
+			installConfig.Config.Platform.Ovirt.NetworkName,
+			installConfig.Config.Platform.Ovirt.VNICProfileID,
+			string(*rhcosImage),
+			clusterID.InfraID,
+			masters[0].Spec.ProviderSpec.Value.Object.(*ovirtprovider.OvirtMachineProviderSpec),
+		)
+		if err != nil {
+			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
+		}
+		t.FileList = append(t.FileList, &asset.File{
+			Filename: fmt.Sprintf(TfPlatformVarsFileName, platform),
+			Data:     data,
+		})
+	case vsphere.Name:
+		controlPlanes, err := mastersAsset.Machines()
+		if err != nil {
+			return err
+		}
+		controlPlaneConfigs := make([]*vsphereprovider.VSphereMachineProviderSpec, len(controlPlanes))
+		for i, c := range controlPlanes {
+			controlPlaneConfigs[i] = c.Spec.ProviderSpec.Value.Object.(*vsphereprovider.VSphereMachineProviderSpec)
+		}
+
+		// Set this flag to use an existing folder specified in the install-config. Otherwise, create one.
+		preexistingFolder := installConfig.Config.Platform.VSphere.Folder != ""
+
+		data, err = vspheretfvars.TFVars(
+			vspheretfvars.TFVarsSources{
+				ControlPlaneConfigs: controlPlaneConfigs,
+				Username:            installConfig.Config.VSphere.Username,
+				Password:            installConfig.Config.VSphere.Password,
+				Cluster:             installConfig.Config.VSphere.Cluster,
+				ImageURL:            string(*rhcosImage),
+				PreexistingFolder:   preexistingFolder,
+			},
+		)
+		if err != nil {
+			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
+		}
+		t.FileList = append(t.FileList, &asset.File{
+			Filename: fmt.Sprintf(TfPlatformVarsFileName, platform),
+			Data:     data,
+		})
+	case kubevirt.Name:
+		masters, err := mastersAsset.Machines()
+		if err != nil {
+			return err
+		}
+		masterSpecs := make([]*kubevirtprovider.KubevirtMachineProviderSpec, len(masters))
+		for i, m := range masters {
+			masterSpecs[i] = m.Spec.ProviderSpec.Value.Object.(*kubevirtprovider.KubevirtMachineProviderSpec)
+		}
+
+		labels := kubevirtutils.BuildLabels(clusterID.InfraID)
+		data, err := kubevirttfvars.TFVars(
+			kubevirttfvars.TFVarsSources{
+				MasterSpecs:     masterSpecs,
+				ImageURL:        string(*rhcosImage),
+				Namespace:       installConfig.Config.Kubevirt.Namespace,
+				ResourcesLabels: labels,
+			},
+		)
+		if err != nil {
+			return errors.Wrapf(err, "failed to get %s Terraform variables", platform)
+		}
+		t.FileList = append(t.FileList, &asset.File{
+			Filename: fmt.Sprintf(TfPlatformVarsFileName, platform),
+			Data:     data,
+		})
+
 	default:
 		logrus.Warnf("unrecognized platform %s", platform)
 	}
@@ -452,7 +612,7 @@ func injectInstallInfo(bootstrap []byte) (string, error) {
 
 	config.Storage.Files = append(config.Storage.Files, ignition.FileFromString("/opt/openshift/manifests/openshift-install.yaml", "root", 0644, cm))
 
-	ign, err := json.Marshal(config)
+	ign, err := ignition.Marshal(config)
 	if err != nil {
 		return "", errors.Wrap(err, "failed to marshal bootstrap Ignition config")
 	}
